@@ -25,6 +25,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <string.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -240,7 +241,7 @@ char *_alpm_strreplace(const char *str, const char *needle, const char *replace)
 /* Create a lock file */
 int _alpm_lckmk()
 {
-	int fd, count = 0;
+	int fd;
 	char *dir, *ptr;
 	const char *file = alpm_option_get_lockfile();
 
@@ -251,17 +252,9 @@ int _alpm_lckmk()
 		*ptr = '\0';
 	}
 	_alpm_makepath(dir);
-
-	while((fd = open(file, O_WRONLY | O_CREAT | O_EXCL, 0000)) == -1 && errno == EACCES) {
-		if(++count < 1) {
-			sleep(1);
-		}	else {
-			return(-1);
-		}
-	}
-
 	FREE(dir);
 
+	fd = open(file, O_WRONLY | O_CREAT | O_EXCL, 0000);
 	return(fd > 0 ? fd : -1);
 }
 
@@ -401,12 +394,18 @@ int _alpm_rmrf(const char *path)
 	return(0);
 }
 
-int _alpm_logaction(unsigned short usesyslog, FILE *f, const char *fmt, va_list args)
+int _alpm_logaction(unsigned short usesyslog, FILE *f,
+		const char *fmt, va_list args)
 {
 	int ret = 0;
 
 	if(usesyslog) {
-		vsyslog(LOG_WARNING, fmt, args);
+		/* we can't use a va_list more than once, so we need to copy it
+		 * so we can use the original when calling vfprintf below. */
+		va_list args_syslog;
+		va_copy(args_syslog, args);
+		vsyslog(LOG_WARNING, fmt, args_syslog);
+		va_end(args_syslog);
 	}
 
 	if(f) {
