@@ -139,8 +139,8 @@ static void fill_progress(const int bar_percent, const int disp_percent,
 	}
 	/* print percent after progress bar */
 	if(proglen > 5) {
-		/* use disp_percent if it is not 0, else show bar_percent */
-		int p = disp_percent ? disp_percent : bar_percent;
+		/* show total download percent if option is enabled */
+		int p = config->totaldownload ? disp_percent : bar_percent;
 		printf(" %3d%%", p);
 	}
 
@@ -181,6 +181,7 @@ void cb_trans_evt(pmtransevt_t event, void *data1, void *data2)
 			alpm_logaction("installed %s (%s)\n",
 			         alpm_pkg_get_name(data1),
 			         alpm_pkg_get_version(data1));
+			display_optdepends(data1);
 			break;
 		case PM_TRANS_EVT_REMOVE_START:
 			if(config->noprogressbar) {
@@ -202,6 +203,7 @@ void cb_trans_evt(pmtransevt_t event, void *data1, void *data2)
 			         (char *)alpm_pkg_get_name(data1),
 			         (char *)alpm_pkg_get_version(data2),
 			         (char *)alpm_pkg_get_version(data1));
+			display_optdepends(data1);
 			break;
 		case PM_TRANS_EVT_INTEGRITY_START:
 			printf(_("checking package integrity...\n"));
@@ -442,7 +444,10 @@ void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 	int file_percent = 0, total_percent = 0;
 	char rate_size = 'K', xfered_size = 'K';
 
-	if(config->noprogressbar) {
+	if(config->noprogressbar || file_total == -1) {
+		if(file_xfered == 0) {
+			printf(_("downloading %s...\n"), filename);
+		}
 		return;
 	}
 
