@@ -71,14 +71,15 @@ static void usage(int op, const char * const myname)
 
 	if(op == PM_OP_MAIN) {
 		printf("%s:  %s <%s> [...]\n", str_usg, myname, str_opr);
-		printf("%s:\n", str_opt);
+		printf(_("operations:\n"));
 		printf("    %s {-h --help}\n", myname);
 		printf("    %s {-V --version}\n", myname);
 		printf("    %s {-Q --query}   [%s] [%s]\n", myname, str_opt, str_pkg);
 		printf("    %s {-R --remove}  [%s] <%s>\n", myname, str_opt, str_pkg);
 		printf("    %s {-S --sync}    [%s] [%s]\n", myname, str_opt, str_pkg);
 		printf("    %s {-U --upgrade} [%s] <%s>\n", myname, str_opt, str_file);
-		printf(_("\nuse '%s --help' with other options for more syntax\n"), myname);
+		printf(_("\nuse '%s {-h --help}' with an operation for available options\n"),
+				myname);
 	} else {
 		if(op == PM_OP_REMOVE) {
 			printf("%s:  %s {-R --remove} [%s] <%s>\n", str_usg, myname, str_opt, str_pkg);
@@ -129,11 +130,11 @@ static void usage(int op, const char * const myname)
 			printf(_("  -u, --sysupgrade     upgrade all packages that are out of date\n"));
 			printf(_("  -w, --downloadonly   download packages but do not install/upgrade anything\n"));
 			printf(_("  -y, --refresh        download fresh package databases from the server\n"));
-			printf(_("      --needed         only upgrade outdated or not yet installed packages\n"));
+			printf(_("      --needed         don't reinstall up to date packages\n"));
 			printf(_("      --ignore <pkg>   ignore a package upgrade (can be used more than once)\n"));
 			printf(_("      --ignoregroup <grp>\n"
 			         "                       ignore a group upgrade (can be used more than once)\n"));
-			printf(_("  -q  --quiet          show less information for query and search\n"));
+			printf(_("  -q, --quiet          show less information for query and search\n"));
 		}
 		printf(_("      --config <path>  set an alternate configuration file\n"));
 		printf(_("      --logfile <path> set an alternate log file\n"));
@@ -634,7 +635,7 @@ static int _parseconfig(const char *file, const char *givensection,
 			}
 			if(ptr == NULL && strcmp(section, "options") == 0) {
 				/* directives without settings, all in [options] */
-				if(strcmp(key, "NoPassiveFTP") == 0) {
+				if(strcmp(key, "NoPassiveFtp") == 0) {
 					alpm_option_set_nopassiveftp(1);
 					pm_printf(PM_LOG_DEBUG, "config: nopassiveftp\n");
 				} else if(strcmp(key, "UseSyslog") == 0) {
